!===========
program main
!===========
!
! Appel aux modules

        use mpi
        use mod_dataproc
        use mod_data
        use mod_coefmat
        use mod_secmbre
        use mod_gradient
        use mod_pdscal
        use mod_imprim
!
        implicit none
!
! Declaration des tableaux dynamiques
        real(kind=8), dimension(:,:), allocatable :: A
        real(kind=8), dimension(:), allocatable :: b, u, u0, uexact
!
! Declaration des autres variables
        real(kind=8) :: xi
        integer :: nx, i, ncount
        character(len=99) :: charnitert, charniterx
        character(len=99) :: charnxg
        character(len=99), dimension(0:1000) :: charnxl
!
! Initialisation de MPI
        call MPI_INIT (code)
        call MPI_COMM_RANK (MPI_COMM_WORLD, moi, code)
        call MPI_COMM_SIZE (MPI_COMM_WORLD, nprocs, code)
!
! Lecture des donnees
        if (moi==0) then
        open (unit = 10, file = 'datafile', status = 'old')
        read (10,'(a99)') dirloc
        read (10,*) temps, dt
        read (10,*) epst
        read (10,*) nx
        read (10,*) a1, a2, a3
        read (10,*) epsx, niterxmax
        read (10,*) nimp
        close (10)
        end if
!
! Distribution des donnees par moi=0 sur tous les autres processus
        call MPI_BCAST (dirloc, 99, MPI_CHARACTER, 0, MPI_COMM_WORLD, code)
        call MPI_BCAST (temps, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, code)
        call MPI_BCAST (dt, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, code)
        call MPI_BCAST (epst, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, code)
        call MPI_BCAST (nx, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, code)
        call MPI_BCAST (a1, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, code)
        call MPI_BCAST (a2, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, code)
        call MPI_BCAST (a3, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, code)
        call MPI_BCAST (epsx, 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, code)
        call MPI_BCAST (niterxmax, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, code)
        call MPI_BCAST (nimp, 1, MPI_INTEGER, 0, MPI_COMM_WORLD, code)
!
! Pas de maillage sur le domaine global en espace
        dx = 1./float(nx)
!
! Nombre global de noeuds interieurs 1 <= i <= nxg
        nxg = nx-1
        if (moi==0) write(charnxg,*) nxg

        call MPI_BCAST (charnxg, 99, MPI_CHARACTER, 0, MPI_COMM_WORLD, code)
!
! Nombre local de noeuds interieurs par sous-domaine 1 <= i <= nxl
        nxl1 = nint (float(nxg)/float(nprocs))
        nxl2 = nxg - (nxl1*(nprocs-1))
        nxl = nxl1
        if (moi==nprocs-1) nxl = nxl2
        write(charnxl(moi),*) nxl
        write(6,'(a, i3, 4a)') &
              'moi=',moi, &
              '  nombre global de noeuds=',trim(adjustl(charnxg)), &
              '  nombre local de noeuds=', trim(adjustl(charnxl(moi)))
!
! Allocation dynamique des tableaux
        allocate (A(3,nxl), b(nxl), u(nxl), u0(nxl), uexact(nxl))
!
! Solution analytique
        do i = 1, nxl
         xi = (i+(nxl1*moi))*dx
         uexact(i) = 6*xi*(1-xi)
        end do
!
! Matrice du systeme lineaire
        call coefmat (A)
!
! Nombre iterations maximun en temps
        nt = int(temps/dt)
        if (dt==1.d20) nt=1
!
! ========================
        u = 0.d0
! Boucle en temps
        do ncount = 1, nt
! ========================
!
          nitert = ncount
          u0 = u
          call secmbre (u0, b)
          u = gc (A, b, u0)
          errsolloc = (u-uexact).scaloc.(u-uexact)
          errstaloc = (u-u0).scaloc.(u-u0)
          call MPI_REDUCE (errsolloc, normerrsol, 1, MPI_DOUBLE_PRECISION, MPI_SUM, 0, MPI_COMM_WORLD, code)
          call MPI_REDUCE (errstaloc, normerrsta, 1, MPI_DOUBLE_PRECISION, MPI_SUM, 0, MPI_COMM_WORLD, code)
          if (moi==0) then
           normerrsol = dsqrt(dx) * sqrt(normerrsol)
           normerrsta = (dsqrt(dx)/dt) * sqrt(normerrsta)
           write(charnitert,*) nitert
           write(charniterx,*) niterx
          end if
          if ((moi==0).and.(mod(nitert,nimp)==0)) then
           write(6,'(4a, 3(a,d13.7))') &
                'niter.temps=',trim(adjustl(charnitert)), '  niter.gradient=', trim(adjustl(charniterx)), &
                '  residugradient=',normresx, '  err.sol=', normerrsol, '  err.sta=',normerrsta,''
          end if
          if (niterx==1) exit
!
! ===================
        end do
! Fin boucle en temps
! ===================
!
        if (moi==0) then
         write(6,'(5a, 2(a,d13.7))') &
              'FIN CALCUL A CONVERGENCE', &
              '  niter.temps=',trim(adjustl(charnitert)), &
              '  niter.gradient au dernier pas de temps=', trim(adjustl(charniterx)), &
              '  err.sol=', normerrsol, '  err.sta=',normerrsta,''
        end if
        call imprim (u, uexact)
!
! Desallocation des tableaux
        deallocate (A, b, u, u0, uexact)

! Fin de MPI
        call MPI_FINALIZE (code)
!
!===============
end program main
!===============
